//==============================================================================
// aq_class_attituderecord.c
//
// 
// 17 May 2001 Version 1.0
//
// Purpose:
// 	This class holds attitude data records to be written.
//
// Note:
//	See SDP Toolkit 333-CD-600-001 Appendix L, Ephemeris and Attitude File
//	Formats.
//
//      Input Time is TAI Julian Date.   
//	See Theoretical Basis of the SDP Toolkit Geolocation Package for the
//	ECS Project, 445-TP-002-002, May 1995, Section 4.6.3,
//	Section 4.8.1, page 4-19
//
//==============================================================================

#include <string.h>
#include <assert.h>

#include "aq_class_attituderecord.h"
#include "aq_typdef_header.h"
#include "aq_proc_formatconvert_x.h"	// machine-dependent format conversion
#include "constlib_TimeConstants.h"

// 2448988.5003125 = JD midnight 01 Jan 1993 UTC (see SDP page 4-19)
static const double Jan1993 = time_const_01Jan1993_SDP;

//------------------------------------------------------------------------
// Constructor
//
//------------------------------------------------------------------------

attituderecord::attituderecord() : storebase(64)
{  }


//------------------------------------------------------------------------
// Time-
//      beg = 00  end = 07  cnt = 08    double TAI93 time
//
//	2448988.5003125 = JD midnight 01 Jan 1993 UTC (see SDP page 4-19)
//	Input must be TAI Julian Date.
//
//------------------------------------------------------------------------

attituderecord&  attituderecord::Time( double JDtime )
{
    FormatConvert_float8( (JDtime-Jan1993)*86400.0, DataPtr() );
    return *this;
}


double  attituderecord::Time() const
{
    return FormatConvert_float8( DataPtr() )/86400.0 + Jan1993;
}


//page
//------------------------------------------------------------------------
// EulerAngles-
//	beg = 08  end = 31  cnt = 24    double 3 Euler angles
//
//
//------------------------------------------------------------------------

    
attituderecord&  attituderecord::EulerAngles( carray& angles )
{
    assert( angles.size()==3 );

    FormatConvert_float8( angles[0], DataPtr()+8  );
    FormatConvert_float8( angles[1], DataPtr()+16 );
    FormatConvert_float8( angles[2], DataPtr()+24 );

    return *this;	
}


carray attituderecord::EulerAngles() const
{
    array angles(3);

    angles[0] = FormatConvert_float8( DataPtr()+8  );
    angles[1] = FormatConvert_float8( DataPtr()+16 );
    angles[2] = FormatConvert_float8( DataPtr()+24 );

    return angles;
}


//page
//------------------------------------------------------------------------
// AngleVelocities-
//              beg = 32  end = 55  cnt = 24    Position vector
//
//
//
//------------------------------------------------------------------------

attituderecord&  attituderecord::AngleVelocities( carray& avel )
{
    assert( avel.size()==3 );

    FormatConvert_float8( avel[0], DataPtr()+32 );
    FormatConvert_float8( avel[1], DataPtr()+40 );
    FormatConvert_float8( avel[2], DataPtr()+48 );

    return *this;	
}


carray attituderecord::AngleVelocities() const
{
    array vel(3);

    vel[0] = FormatConvert_float8( DataPtr()+32 );
    vel[1] = FormatConvert_float8( DataPtr()+40 );
    vel[2] = FormatConvert_float8( DataPtr()+48 );

    return vel;
}


//------------------------------------------------------------------------
// Quality Flag
//              beg = 56  end = 59  cnt = 4    
//
//------------------------------------------------------------------------
    
uint attituderecord::QaFlag() const
{
    return  FormatConvert_ulong4( DataPtr()+56 );
}


attituderecord& attituderecord::QaFlag(uint flags)
{
    FormatConvert_long4( flags, DataPtr()+56);
    return *this;
}


//page
//------------------------------------------------------------------------
// Spares
//		beg = 60 end = 63 cnt = 4  
//
// This is for diagnostics only- production version should not read or 
// write to spares
//------------------------------------------------------------------------
 
const char* attituderecord::Spares() const
{
    return DataPtr()+60;
}

char* attituderecord::Spares() 
{
    return DataPtr()+60;
}







