//==============================================================================
// aq_class_ephemerisfile.c
//
// 
// 21 May 2001 Version 1.0
//
// Purpose:
// 	This class defines an ephemeris output file in SDP format
//
// Note:
//	Headers are written when file is closed, so headers can be updated any
//	time before closure.  Headers passed to file constructor must not be
//	destroyed before file closure.
//
//	See SDP Toolkit 333-CD-600-001 Appendix L, Ephemeris and Attitude File
//	Formats.
//
// 	General format:
//	SDP Header
//	ephemeris subheader
//	ephemeris records
//	metadata  records
//
//==============================================================================

#include  "aq_class_ephemerisfile.h"


//-------------------------------------------------------------------------------
// Constructor
//
//	write initial headers (updated later in destructor)
//-------------------------------------------------------------------------------


ephemerisfile::ephemerisfile
( 
    const string& fileName, 
    const commonSDPheader&     comHeader,
    const ephemerissubheader&  ephHeader
)   
   : ofstream( fileName.c_str() ), 
     basefile( fileName, "ephemeris" ),
     _comHeader(comHeader),
     _ephHeader(ephHeader)
{
    _comHeader.lock();
    _ephHeader.lock();
 
    ofstream::write( comHeader.DataPtr(), comHeader.DataSize() );
    ofstream::write( ephHeader.DataPtr(), ephHeader.DataSize() );
}


//-------------------------------------------------------------------------------
// Write Ephemeris Records
//
//-------------------------------------------------------------------------------

ephemerisfile& ephemerisfile::write(const ephemerisrecord& ephRcd )
{
    ofstream::write( ephRcd.DataPtr(), ephRcd.DataSize() );
    return *this;
}


//-------------------------------------------------------------------------------
// Write MetaData Records
//
//-------------------------------------------------------------------------------

ephemerisfile& ephemerisfile::write(const ephemerismetadata& metaRcd )
{
    ofstream::write( metaRcd.DataPtr(), metaRcd.DataSize() );
    return *this;
}



//-------------------------------------------------------------------------------
// Close
//
// write footer and update headers
//-------------------------------------------------------------------------------

void ephemerisfile::close()
{ 
    seekp(0);
    ofstream::write( _comHeader.DataPtr(), _comHeader.DataSize() );
    ofstream::write( _ephHeader.DataPtr(), _ephHeader.DataSize() );
}


//-------------------------------------------------------------------------------
// Destructor
//
//-------------------------------------------------------------------------------

ephemerisfile::~ephemerisfile()
{
    close();

    _comHeader.unlock();
    _ephHeader.unlock();
}





