//==============================================================================
// aq_class_ephemerisrecord.c
//
// 
// 17 May 2001 Version 1.0
//
// Purpose:
// 	This class holds ephemeris data records to be written.
//
//	See SDP Toolkit 333-CD-600-001 Appendix L, Ephemeris and Attitude File
//	Formats.
//
// 	General format:
//	SDP Header
//	ephemeris subheader
//	ephemeris records
//	metadata  records
//
//==============================================================================

#include <string.h>
#include <assert.h>


#include "aq_class_ephemerisrecord.h"
#include "aq_typdef_header.h"
#include "aq_proc_formatconvert_x.h"
#include "constlib_TimeConstants.h"

static const double Jan1993 = time_const_01Jan1993_SDP;


//------------------------------------------------------------------------
// Constructor
//
//------------------------------------------------------------------------

ephemerisrecord::ephemerisrecord() : storebase(64)
{  }


//------------------------------------------------------------------------
// time-
//   	beg = 00  end = 07  cnt = 08    double TAI93 time
//	
//------------------------------------------------------------------------

ephemerisrecord&  ephemerisrecord::time( double JDtime )
    {
	FormatConvert_float8( (JDtime-Jan1993)*86400.0, DataPtr() );
	return *this;
    }


double ephemerisrecord::time( ) const
{
    return FormatConvert_float8( DataPtr() )/86400.0 + Jan1993;
}


//page
//------------------------------------------------------------------------
// Position-
//              beg = 08  end = 31  cnt = 24    Position vector
//
//
//------------------------------------------------------------------------

    
ephemerisrecord&  ephemerisrecord::Position( carray& pos )
    {
	assert( pos.size()==3 );

	FormatConvert_float8( pos[0], DataPtr()+8  );
	FormatConvert_float8( pos[1], DataPtr()+16 );
	FormatConvert_float8( pos[2], DataPtr()+24 );

	return *this;	
    }


carray ephemerisrecord::Position() const
{
    array pos(3);
	
    pos[0] = FormatConvert_float8( DataPtr()+8  );
    pos[1] = FormatConvert_float8( DataPtr()+16  );
    pos[2] = FormatConvert_float8( DataPtr()+24  );

    return pos;
}



//page
//------------------------------------------------------------------------
// Velocity-
//              beg = 32  end = 55  cnt = 24    Position vector
//
//------------------------------------------------------------------------

ephemerisrecord&  ephemerisrecord::Velocity( carray& vel )
{
	assert( vel.size()==3 );

	FormatConvert_float8( vel[0], DataPtr()+32 );
	FormatConvert_float8( vel[1], DataPtr()+40 );
	FormatConvert_float8( vel[2], DataPtr()+48 );

	return *this;	
}


carray ephemerisrecord::Velocity() const
{
    array vel(3);
	
    vel[0] = FormatConvert_float8( DataPtr()+32  );
    vel[1] = FormatConvert_float8( DataPtr()+40  );
    vel[2] = FormatConvert_float8( DataPtr()+48  );

    return vel;
}


//------------------------------------------------------------------------
// Quality Flag
//              beg = 56  end = 59  cnt = 4    
//
//------------------------------------------------------------------------
    
uint ephemerisrecord::QaFlag() const
{
    return  FormatConvert_ulong4( DataPtr()+56 );
}

    
ephemerisrecord&  ephemerisrecord::QaFlag( uint flags )
{
    FormatConvert_long4( flags,  DataPtr()+56 );
    return *this;
}



//page
//------------------------------------------------------------------------
// Spares
//		beg = 60 end = 63 cnt = 4
//
// This is for diagnostics only- production version should not read or
// write to spares
//------------------------------------------------------------------------

const char* ephemerisrecord::Spares() const
{
    return DataPtr()+60;
}

char* ephemerisrecord::Spares()
{
    return DataPtr()+60;
}






