//==============================================================================
// aq_class_ephemerissubheader.c
//
// 
// 18 May 2001 Version 1.0
//
// Purpose:
// 	This class defines portion of SDP Header Standard Sructure specific to 
// 	the ephemeris header.
//
// Note:
// 	General format:
//	SDP Header
//	ephemeris subheader
//	ephemeris records
//	metadata  records
//
//	See SDP Toolkit 333-CD-600-001 Appendix L, Ephemeris and Attitude File
//	Formats.
//
//==============================================================================

#include <assert.h>
#include "aq_class_ephemerissubheader.h"
#include "aq_proc_formatconvert_x.h"		// machine-dependent formats

//------------------------------------------------------------------------------
// Constructor
//
//------------------------------------------------------------------------------

ephemerissubheader::ephemerissubheader() : storebase(372)
{
    ptr = DataPtr(); 
}

    
//------------------------------------------------------------------------------
// Orbits
//              beg = 00  end = 03  cnt = 04   	uint number of orbits
//              beg = 04  end = 07  cnt = 04   	uint begin orbit number
//              beg = 08  end = 11  cnt = 04   	uint end   orbit number
//
//------------------------------------------------------------------------------

ephemerissubheader& 
ephemerissubheader::Orbits( uint begOrbit, uint endOrbit, uint orbitCnt )
{
    FormatConvert_long4( orbitCnt, ptr  );
    FormatConvert_long4( begOrbit, ptr+4);
    FormatConvert_long4( endOrbit, ptr+8);

    return *this;
}


vector<uint> ephemerissubheader::Orbits() const
{
    vector<uint> orbits(3);

    orbits[2] = FormatConvert_long4( ptr   );	// orbit count
    orbits[0] = FormatConvert_long4( ptr+4 );	// beg orbit
    orbits[1] = FormatConvert_long4( ptr+8 );	// end orbit

    return orbits;
}


//page
//------------------------------------------------------------------------------
// RefFrame
//              beg = 12  end = 19  cnt = 08   	
//
//------------------------------------------------------------------------------
    
ephemerissubheader& 
ephemerissubheader::RefFrame( const string& rframe )
{
    rframe.copy( &ptr[12], 8 );
    return *this;
}


string ephemerissubheader::RefFrame() const
{
    return string( DataPtr()+12, 8 );
}


//------------------------------------------------------------------------------
// KeplerElements
//              beg = 20  end = 67  cnt = 48   	
//
//------------------------------------------------------------------------------

ephemerissubheader& 
ephemerissubheader::KeplerElements( carray kepler )
{
    assert( kepler.size() == 6);
    char  *ptr = DataPtr() + 20;

    for(size_t i = 0; i<6; i++)  
	FormatConvert_float8( kepler[i], ptr+i*8 );	

    return *this;
}


carray ephemerissubheader::KeplerElements( ) const
{
    array kel(6);
    char  *ptr = DataPtr() + 20;

    for(size_t i = 0; i<6; i++)  
	kel[i] = FormatConvert_float8( ptr+i*8 );	

    return kel;
}


//page
//------------------------------------------------------------------------------
// EpochTimeTAI
//		beg = 68  end = 75 cnt = 8
//
//------------------------------------------------------------------------------

ephemerissubheader& ephemerissubheader::EpochTimeTAI( double time )
{
	if(time < 0)
		FormatConvert_float8( -999.999999, DataPtr()+68);
	else
    	FormatConvert_float8( time, DataPtr()+68 );
    return *this;
}

double ephemerissubheader::EpochTimeTAI() const
{
    return FormatConvert_float8( DataPtr()+68 );
}


//page
//------------------------------------------------------------------------------
// qaParameters
//              beg = 76  end = 139  cnt = 64   	
//
//------------------------------------------------------------------------------

 vector<float> ephemerissubheader::qaParameters() const
{
    vector<float> qap(16);
    char  *ptr = DataPtr() + 76;

    for(size_t i = 0; i<16; i++)  
	qap[i] = FormatConvert_float4( ptr+i*4 );	

    return qap;
}


//------------------------------------------------------------------------------
// qaStatistics
//              beg = 140  end = 155  cnt = 16   	
//
//------------------------------------------------------------------------------

 vector<float> ephemerissubheader::qaStatistics() const
{
    vector<float> stat(4);
    char  *ptr = DataPtr() + 140;

    for(size_t i = 0; i<4; i++)  
	stat[i] = FormatConvert_float4( ptr+i*4 );	

    return stat;
}


