//==============================================================================
// aq_class_gbadpacket.c
//
// 
// 17 May 2001 Version 1.0
// 26 Jun 2002         1.1 add Fine(), change fine ushort to ushort2
// 27 Oct 2003 Version 1.2 - allow APID to change
//
// Purpose:
//     This class defines Aqua CCDS Ground Based Attitude Determination Data 
//     (GBAD) Packet APID 957. These are input packets to program. 
//
// Note:
//	See EOS PM-1 Spacecraft to EOS Ground Systems ICD, GSFC 422-11-19-03, 
//	and Time Code Formats, CCSDS 301.0-B-2, Blue Book, Issue 2 April 1990.
//
//      ItemList contains the following entries:
//	p[ 0] - p[ 2] 	position
//	p[ 3] - p[ 5] 	velocity
//	p[ 6] - p[ 9] 	attitude
//	p[10] - p[12]   attitude rate
//
//	Time returned is TAI Julian Date from 01 Jan 1953 at 0 hr =   
//	2436203.5. See CCSDS Recommendation for Time Code Formats
//
//	See Theoretical Basis of the SDP Toolkit Geolocation Package for the
//	ECS Project, 445-TP-002-002, May 1995,  Section 4.8.1
//
//==============================================================================

#include <stddef.h>
#include <iomanip>

#include "aq_class_gbadpacket.h"
#include "aq_proc_printarray.h"
#include "aq_proc_formatconvert_x.h"	    // machine-dependent formats
#include "aq_proc_Mil1750_EPFP.h"	    	// squirrely star tracker float fmt.
#include "constlib_TimeConstants.h"

size_t gbadpacket::pktsize = 6+120;   	    // CCSDs headr  + data zone
					    					// Aqua is default

size_t gbadpacket::apid = 957;	    		// Application ID of GBAD Packet

static const double Jan1958 = time_const_01Jan1958_TAI;


//------------------------------------------------------------------------------
// Constructor
//	
//------------------------------------------------------------------------------

gbadpacket::gbadpacket() : packet(pktsize), p(13), time(0.)
{ }


//page
//------------------------------------------------------------------------------
// ReadPkt-
//
//------------------------------------------------------------------------------

istream& gbadpacket::ReadPkt(istream& is)
{
    is.read( DataPtr(), DataSize() );
    char *tPtr;

    // -------------------------------------------------------------------------
    // time  (See header notes)

    tPtr = DataPtr() + 6;	// time start

    size_t coarse = FormatConvert_long4 ( tPtr+2 );  		// seconds
    unsigned short fine = FormatConvert_ushort2( tPtr+6 ); 

    time = (coarse + fine*15.2e-6) / 86400.0 + Jan1958;


    // -------------------------------------------------------------------------
    // position, velocity, attitude 

    tPtr = DataPtr() + 6 + 8;	// data start
    for(size_t i = 0; i<10; i++) p[i] = MiL1750_EPFP( tPtr + i*6 );


    // -------------------------------------------------------------------------
    // angle rate

    tPtr = DataPtr() + 6 + 8 + 60;
    for(size_t i = 0; i<3; i++) p[10+i] = MiL1750_FP( tPtr + i*4 );

    return is;
}


//page
//------------------------------------------------------------------------------
// TimeTag
//	packet Time Tag returned in Julian Days. 01 Jan 1958 = JD 2436203.5
//------------------------------------------------------------------------------

double gbadpacket::TimeTag() const
{
    return time;
}

gbadpacket& gbadpacket::TimeTag( double _time )
{
    time = _time;
    return *this;
}


size_t gbadpacket::CoarseTime() const
{
    char*  tPtr = DataPtr() + 6;			// time start
    return FormatConvert_long4 ( tPtr+2 );  		// seconds
}


size_t gbadpacket::FineTime() const
{
    char*  tPtr = DataPtr() + 6;			// time fraction of
     return  FormatConvert_ushort2( tPtr+6 );		// second
}


//------------------------------------------------------------------------------
// Position
//	
//------------------------------------------------------------------------------
    
carray gbadpacket::Position() const 
{
    return vector<double>( p.begin(), p.begin()+3 );
}


//------------------------------------------------------------------------------
// Velocity
//	
//------------------------------------------------------------------------------

carray gbadpacket::Velocity() const
{
    return vector<double>( p.begin()+3, p.begin()+6 );
}


//------------------------------------------------------------------------------
// Attitude
//	
//------------------------------------------------------------------------------

carray gbadpacket::Attitude() const
{
    return vector<double>( p.begin()+6, p.begin()+10 );
}


//page
//------------------------------------------------------------------------------
// AngleRates
//	
//------------------------------------------------------------------------------

carray  gbadpacket::AngleRates() const
{
    return vector<double>(p.begin()+10, p.begin()+13);
}


//------------------------------------------------------------------------------
// itemlist
//
//------------------------------------------------------------------------------

array& gbadpacket::ItemList() 
{
    return p;
}


carray& gbadpacket::ItemList() const
{
    return p;
}


//------------------------------------------------------------------------------
// operator<<
//	
//------------------------------------------------------------------------------

ostream& operator<<( ostream& os, const gbadpacket& pkt)
{
    os<<(const packet)pkt;

    os.setf( ios::fixed );
    os.precision(8);

    os<<"\ngbad_timetag   = "<<pkt.TimeTag()
      <<"  TAI seconds = "<<pkt.CoarseTime()
      <<"\ngbad_position  = "<<pkt.Position()
      <<"\ngbad_velocity  = "<<pkt.Velocity()
      <<"\ngbad_attitude  = "<<pkt.Attitude()
      <<"\ngbad_att_rates = "<<pkt.AngleRates()
      <<flush;

    return os;
}














