//==============================================================================
// aq_class_packetfile.c
//
// 
// 17 May 2001 Version 1.0
// 13 Aug 2002 Version 2.0 - redesign to handle multiple APIDs. NOTE CHANGE IN
//			     CONSTRUCTOR ARGS.
//
// 02 Apr 2003 Version 2.1 - ignore packets with duplicate sequence numbers
// 11 Aug 2003 Version 2.2 - at NoPackets() function
//
// This class defines a CCSDS Packet File
// The index is the packet number in the file. First packet is 0.
//
//==============================================================================

#include "aq_class_packetfile.h"


//-----------------------------------------------------------------------------
// Constructor
//
//	Build a list of packet locations in file for specified APID. If error,
//	EndPacket() should point to last good packet of requested type. 
//-----------------------------------------------------------------------------


packetfile::packetfile( const string& fileName, size_t apid )

: ifstream ( fileName.c_str() ), 
  basefile( fileName, "packet" )
{
    int lastSequenceNo = -1;
    pktAPID =  apid;
    index   = -1;
    packet  packet(6);				// CCSDS Header only


    if( !*this ) return;			// file open failed

    while(1)
    {
	size_t cpos = tellg();			// get packet start

	if( !packet.ReadPkt(*this) ) break;	// read pkt header
   	seekg( packet.PktSize(), ios::cur );	// read past rest of pkt

						
	if( packet.Version()  != 0  ||		// sanity checks
	    packet.PktSize()  <  1  ||	
	    packet.Sequence() <  0 
	  ) 
	  {
		setstate( ios::badbit );
		return;
	  }

	if( packet.APID() == pktAPID ) {
	    if( packet.Sequence() != lastSequenceNo)  pktList.push_back( cpos );
	    lastSequenceNo = packet.Sequence();
	}
    }

    if( eof() ) clear();			// EOF is expected to terminate
}						// loop


//page
//-----------------------------------------------------------------------------
// BegPacket
//
//	return index of first packet ( always 0 )
//-----------------------------------------------------------------------------

size_t  packetfile::BegPacket()  const
{
    return 0;
}


//-----------------------------------------------------------------------------
// EndPacket
//
//	return index of last packet
//-----------------------------------------------------------------------------

size_t  packetfile::EndPacket()  const
{
    if( pktList.size() ) return pktList.size() - 1;
    return 0;
}


//-----------------------------------------------------------------------------
// CurPacket
//
// 	return index of current (just read) packet. 
//-----------------------------------------------------------------------------

size_t  packetfile::CurPacket()  const
{
    return (size_t)index;
}


//-----------------------------------------------------------------------------
// NoPackets
//
// number of specified packets in file
//-----------------------------------------------------------------------------

size_t  packetfile::NoPackets() const
{
	return pktList.size();
}

//page
//-----------------------------------------------------------------------------
// ReadPacket
//
//-----------------------------------------------------------------------------

packetfile&  packetfile::ReadPacket( packet& pkt, size_t pktnumber )
{
    index = pktnumber;

    if( pktnumber >= pktList.size() ) 		//end of file
    {
	setstate( ios::eofbit | ios::failbit );	
	return *this;
    }

    seekg( pktList[ index ] );
    pkt.ReadPkt( *this );
						// data packet sizes must agree
    if( pkt.PktSize() + 6 != pkt.DataSize() )  setstate( ios::badbit);
	
    return *this;
}
    

packetfile& packetfile::ReadPacket( packet& pkt )
{
    return ReadPacket( pkt, ++index);
}


//-----------------------------------------------------------------------------
// ReqApid
//
//-----------------------------------------------------------------------------

size_t packetfile::ReqApid() const
{
    return pktAPID;
}


//-----------------------------------------------------------------------------
// operator<<
//
//-----------------------------------------------------------------------------

ostream& operator<<( ostream& os, packetfile& pfile)
{
    os<<"\nfileValid "<<(!pfile == false)
      <<"\nreq APID= "<<pfile.ReqApid()
      <<"\nbeg pkt = "<<pfile.BegPacket() 
      <<"\nend pkt = "<<pfile.EndPacket() 
      <<"\ncur pkt = "<<(int)(pfile.CurPacket())
      <<endl;

    return os;
}
 


