//=============================================================================
// aq_class_windowbuffer.h
//
//  
// 08 Jan 2002 Version 1.0
// 21 Mar 2006 Version 1.1 - 	add explicit stackbuffer<T>:: to method calls, 
//				e.g. stackbuffer<T>::add()
//
// Description:
//    This class adds an index to a stackbuffer (see aq_class_stackbuffer.h).
//    add() is like stackbuffer.add() but decrements the index so that index
//    tends to follow objects as they enter stack at bottom and leave at top.
//
//    get() increments index so that successive adds() and gets() point to the
//    same place in the stack.
// 
//    By controlling the number of add() and get() calls, all objects passing
//    thru the window buffer are pointed to by get() and nearby objects remain
//    accessible for filtering, etc.
//   
//
// template<class T> 
// class windowbuffer : public stackbuffer<T>
// {
// public:
//     windowbuffer( size_t size ) : stackbuffer<T>( size )
//     T* add()		// points to buffer to load object
//     T* get()		// points to buffer to get object. 0 if buf not loaded.
//     T* next()		// next object that will be read. Zero if at end.
//     T* last()		// last object that was read. Zero =if at beginning.
// 
// 			// look in any object. 
//     T* peek ( size_t index ) {return stackbuffer<T>::get( index );}
// 
// 			// index of next buffer to get. 
// 		        // range is 0 to size()-1, -1 at end.
//     int    index( ) const {return windex<length() ? windex : -1;}		
// 
//     bool   atbeg( ) const {return windex == 0;}		// at start of window
//     bool   atend( ) const {return windex >= length();}	// next get will fail
//     void   clear( ) { windex = 0; stackbuffer<T>::clear(); }
//
//     virtual ~windowbuffer( ) {}
// };
//
//
// from base class:
//     bool   full()   const;
//     size_t length() const;		// number of objects in buffer
//     size_t size()   const;		// max no. of objects allowed
//
// ostream& operator<<( ostream& os, windowbuffer<T>& buf )
//
//=============================================================================

#if !defined(AQ_CLASS_WINDOWBUFFER)
#define      AQ_CLASS_WINDOWBUFFER

#include "aq_class_stackbuffer.h"

//page
template<class T> 
class windowbuffer : public stackbuffer<T>
{
    int windex;		// index of buffer used by next get

public:
    explicit windowbuffer( size_t size ) : stackbuffer<T>( size )
    {
	windex = 0;
    }


    T* add()		// points to buffer to load object
    {
	if( !stackbuffer<T>::size() ) return 0;
	if(  stackbuffer<T>::full() ) if(--windex < 0 ) windex = 0;
	return stackbuffer<T>::add( );
    }


    T* get()		// points to buffer to get object. 0 if buf not loaded.
    {
	if( windex >= stackbuffer<T>::length() ) return 0;
	return stackbuffer<T>::peek( windex++ );
    }


    T* next()		// next object that will be read. Zero if at end.
    {
	if( windex >= stackbuffer<T>::length() ) return 0;
	return stackbuffer<T>::peek( windex);	
    }


    T* last()		// last object that was read. Zero =if at beginning.
    {
	if( windex-1 < 0 ) return 0;
	return stackbuffer<T>::peek( windex-1 );	
    }
			// look in any object. 
    T* peek ( size_t index ) {return stackbuffer<T>::peek( index );}
    const T* peek( size_t index ) const { return stackbuffer<T>::peek( index );}

			// index of next buffer to get. 
		        // range is 0 to size()-1, -1 at end.
    int    index( ) const {return windex < stackbuffer<T>::length() ? windex : -1;}		

    bool   atbeg( ) const {return windex == 0;}		// at start of window
    bool   atend( ) const {return windex >= stackbuffer<T>::length();}	// next get will fail
    void   clear( ) { windex = 0; stackbuffer<T>::clear(); }

    virtual ~windowbuffer( ) {}
};


//page
//-----------------------------------------------------------------------------
// operator<<
//	diagnostic print
//-----------------------------------------------------------------------------

template<class T>
static ostream& operator<<( ostream& os, const windowbuffer<T>& buf )
{
    os<<"atbeg="<<buf.atbeg()<<"  atend="<<buf.atend()
         <<"  index="<<setw(2)<<buf.index()<<"  "<<(stackbuffer<T>&)buf;


    return os;
}

#endif








