//==============================================================================
// Attitude File Diagnostic - 
//
// 24 Jul 2001 Version 1.0
// 11 Aug 2003 Version 1.1 - print beg and no of records, add time seq. check
//
// DumpAttitude   -file filename -beg [0] -cnt [all]
//
//==============================================================================

#include <iostream>
#include <fstream>
#include <iomanip>

#include "aq_typdef_header.h"
#include "aq_proc_printarray.h"
#include "aq_proc_fileopencheck.h"
#include "aq_class_commonSDPheader.h"
#include "aq_class_attitudesubheader.h"
#include "aq_class_attituderecord.h"
#include "aq_proc_readcmdline.h"
#include "timelib_class_datetime.h"
#include "aq_class_qualityflags.h"


//------------------------------------------------------------------------------
// cfg_list
//
//------------------------------------------------------------------------------

class cfg_list : public configuration_list
{
public:
    cfg_list()
    {
        define( "-file"  );
        define( "-beg", "0" );
        define( "-cnt", "all" );
    }
};


//------------------------------------------------------------------------
// main
//
//------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    cout<<"\nAttitude File Diagnostic"<<endl;

    cfg_list cmdline;
    ReadCmdLine( cmdline, argc, argv );

    //-----------------------------------
    // Open Ephemeris File
    //-----------------------------------
    string fileName  = cmdline.getvalue( "-file" );

    ifstream attFile( fileName.c_str() );
    if( !attFile )
    {
	cerr<<"\nAttitude File open failed: "<<fileName<<endl;
	return -1;
    }

    cout<<"Attitude File opened: "<<fileName<<endl;


    //-----------------------------------
    // Read  SDP header  
    //-----------------------------------

    commonSDPheader SDPheader;
    attFile.read( SDPheader.DataPtr(), SDPheader.DataSize() );
 
    cout.setf( ios::fixed );
    cout<<endl;

    cout<<"\nSpacecraft = "<<SDPheader.SpacecraftID()
	<<"\nSource     = "<<SDPheader.Source()
	<<"\nVersion    = "<<SDPheader.Version()
        <<"\nTime Range = "<<SDPheader.TimeRange()
        <<"\nInterval   = "<<SDPheader.Interval()
        <<"\nNo. Records= "<<SDPheader.NoRecords()
	<<"\nNo. U. Refs= "<<SDPheader.NoURefs()
	<<endl;


    //-----------------------------------
    // Read  Attitude SubHeader
    //-----------------------------------

    attitudesubheader  attsubhdr;
    attFile.read( attsubhdr.DataPtr(), attsubhdr.DataSize() );

    cout<<endl;
    cout<<"\nAngle order= "
	<<attsubhdr.AngleOrder()[0]<<' '
	<<attsubhdr.AngleOrder()[1]<<' '
	<<attsubhdr.AngleOrder()[2]
	<<endl;

     vector<float> 
     qaP = attsubhdr.qaParameters();
     cout<<"\nQA Parameters";
     cout.setf( ios::showpos );

     for( size_t pi = 0; pi<16; pi++)
	cout<<'\n'<<setw(2)<<pi<<' '<<qaP[pi];
     cout<<endl;

     qaP = attsubhdr.qaStatistics();
     cout<<"\nQA Statistics";

     for( size_t pi = 0; pi<4; pi++)
	cout<<'\n'<<setw(2)<<pi<<' '<<qaP[pi];
     cout<<endl;


    //-----------------------------------
    // No. of Universal References
    //-----------------------------------

    attFile.seekg(SDPheader.NoURefs()*256, ios::cur);


    //-----------------------------------
    // Attitude Records
    //-----------------------------------

    size_t beg = atoi( cmdline.getvalue( "-beg" ).c_str() );

    size_t cnt = size_t(-1);
    size_t knt = 0;

    if( cmdline.getvalue( "-cnt" ) != (string)"all")
	cnt = atoi( cmdline.getvalue( "-cnt" ).c_str() );

    quality_flags qflags;
    attituderecord attRec;

	cout<<"\nBeg Record = "<<beg<<endl;

	if( cnt != size_t(-1) ) cout<<"Rec Count  = "<<cnt<<endl;   // **
		else cout<<"Rec Count = all"<<endl; 

    cout<<"\nJulian-Date  Angle Velocities"<<endl;
    cout.unsetf( ios::showpos );

	double lastTime = 0;
    double thisTime = 0;

    for( size_t nrec = 0; nrec < SDPheader.NoRecords(); nrec++)
    {
	attFile.read( attRec.DataPtr(), attRec.DataSize() );
        if( nrec  <  beg ) continue;
	if( knt++ >= cnt ) break;

        if( attFile.eof())
	{
	    cout<<"Early End of File"<<endl;
	    break;
	}

	cout.precision(8);
	cout<<attRec.Time()<<' ';
	
	thisTime = attRec.Time();
	if( thisTime <= lastTime )
	{
	    cout<<"\nTime Sequence Error "<<endl;
	}
	lastTime = thisTime;

	cout.setf( ios::showpos );
	cout.precision(6);

	cout<<attRec.EulerAngles()<<' '<<attRec.AngleVelocities()
	    <<' '<<hex<<setw(4)<<setfill('0'); 

	qflags.setqualityflags( attRec.QaFlag() );
	if( qflags.getqualityflags().any() ) cout<<'\n'<<qflags;
	cout<<endl;

	cout.unsetf( ios::showpos );
    }

}






