//==============================================================================
// GBAD Packet File Diagnostic - List Packets in Packet File
//
// 04 June 2001 Version 1.0
// 28 Aug  2007 Version 1.1 -fix output formatting error
//
// List packets in an APID 957 GBAD packet file. Keyword-value pairs
// may be in any order.  Defaults are: -cnt all  -sat AQUA -beg 0 -check no
//
// "-check" checks times.  
//
// ListPackets -file filename -beg [0] -cnt [all] -check <yes|no> -sat <AQUA|AURA>
//
//
//==============================================================================

#include <iostream>
#include <fstream>
#include <iomanip>

#include "aq_class_gbadpacket.h"
#include "aq_class_packetfile.h"
#include "aq_typdef_header.h"
#include "aq_proc_printarray.h"
#include "aq_proc_fileopencheck.h"
#include "aq_proc_readcmdline.h"


//------------------------------------------------------------------------------
//  stringistrue
//
//------------------------------------------------------------------------------

    bool stringistrue(string str)
    {
        return  (str=="yes")  || (str=="true") || (str=="1"  ) ||
                (str=="y"  )  || (str=="t"   ) || (str=="ok" ) ||
		(str=="on" );
    }


//------------------------------------------------------------------------------
// cfg_list
//
//------------------------------------------------------------------------------

class cfg_list : public configuration_list
{
public:
    cfg_list()
    {
        define( "-file"  );
        define( "-beg", "0" );
        define( "-cnt", "all" );
	define( "-check", "no" );
	define( "-sat", "AQUA" );
    }
};


//------------------------------------------------------------------------------
// Dump File State
//
//------------------------------------------------------------------------------

void DumpState( packetfile& file )
{
    cout<<"\tEnd Packet = "<<file.EndPacket()<<endl;

    if( file.eof()  )
    {
	 cout<<"\tEOF  Bit Set"<<endl;   
	 return;
    }

    if( file.bad()  ) cout<<"\tBAD  Bit Set"<<endl;   
    if( file.fail() ) cout<<"\tFAIL Bit Set"<<endl;   
}



//------------------------------------------------------------------------
// Debug- not normally used
//
//------------------------------------------------------------------------

void DumpFmtFlags( ios_base::fmtflags  flags )
{
		cout.flags( ios::boolalpha );
		cout<<"\nskipws    = "<<(flags & ios_base::skipws)
		    <<"\nleft      = "<<(flags & ios_base::left)
		    <<"\nright     = "<<(flags & ios_base::right)
		    <<"\ninternal  = "<<(flags & ios_base::internal)
			<<"\ndec       = "<<(flags & ios_base::dec)
			<<"\nhex       = "<<(flags & ios_base::hex)
			<<"\noct       = "<<(flags & ios_base::oct)
			<<"\nscientific= "<<(flags & ios_base::scientific)
			<<"\nfixed     = "<<(flags & ios_base::fixed)
		    <<"\nshowbase  = "<<(flags & ios_base::showbase)
			<<"\nshowpoint = "<<(flags & ios_base::showpoint)
			<<"\nshowpos   = "<<(flags & ios_base::showpos)
		    <<"\nuppercase = "<<(flags & ios_base::uppercase)
		    <<"\nadjustfield "<<(flags & ios_base::adjustfield)
			<<"\nbasefield = "<<(flags & ios_base::basefield)
			<<"\nfloatfield= "<<(flags & ios_base::floatfield)
			<<endl;
		
		cout.flags( flags );
}


//------------------------------------------------------------------------
// main
//
//------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    cout<<"\nPacket File Diagnostic"<<endl;

    cfg_list cmdline;
    ReadCmdLine( cmdline, argc, argv );
    cout<<cmdline<<endl;

    bool check = stringistrue( cmdline.getvalue( "-check" ) );

    string satName  = cmdline.getvalue( "-sat"  );
    if( satName == (string)("AURA") ) gbadpacket::pktsize=214;


    //-----------------------------------
    // Open GBAD File
    //-----------------------------------
 
    string fileName  = cmdline.getvalue( "-file" );

    packetfile pktFile( fileName, gbadpacket::apid );
    FileOpenCheck( pktFile );

    cout<<"Packet File opened: "<<fileName<<endl;
    cout<<"APID "<<gbadpacket::apid
	<<" packet count: "<<pktFile.EndPacket()<<endl;

    if( pktFile.EndPacket() == 0 )
    {
	cout<<"No APID "<<gbadpacket::apid<<" packets found"<<endl;
	return -1;
    }


    //-----------------------------------
    // Read GBAD Packets
    //-----------------------------------

    size_t beg = atoi( cmdline.getvalue( "-beg" ).c_str() );

    size_t cnt = size_t(-1);
    size_t knt = 0;
    short  nrec= 0;

    double lastTime = 0;
    double thisTime = 0;

    if( cmdline.getvalue( "-cnt" ) != (string)"all")
        cnt = atoi( cmdline.getvalue( "-cnt" ).c_str() );

    gbadpacket gPacket;

    cout.setf( ios::fixed );
    cout<<'\n';

    while( 1 )
    {
        if( !pktFile.ReadPacket( gPacket) ) 
	{
	    if( pktFile.eof() ) 
	    {
		cout<<"\nNormal EOF"<<endl;
		break;
	    }

	    cout<<"\nPkt file read error"<<endl;
	    cout<<"\nfile state---"<<pktFile<<endl;
	    cout<<"Pkt. state---"<<gPacket<<endl;
 
	    if( gPacket.PktSize() + 6 != gbadpacket::pktsize )
	    {
		cout<<"\nBad Packet Size"<<endl;
	    }

	    break;
	}

        if( nrec++ <  beg ) continue;
        if( knt++  >= cnt ) break;

	thisTime = gPacket.TimeTag();
  	if( knt>1 && check)
        {
	    if( thisTime >  lastTime + 1.5/86400L ||
	        thisTime <= lastTime
	    ) cout<<"time error ----------------------------"<<endl;
	}	
	lastTime = thisTime;

	cout.setf( ios::right );
	cout.unsetf( ios::showpos );
	cout.fill('0');

	cout<<"\n"<<setw(4)<<nrec
		<<" "<<setw(4)<<gPacket.APID()
		<<" "<<setw(4)<<gPacket.PktSize()
		<<" "<<setw(4)<<gPacket.Sequence()
		<<" "<<thisTime;

	cout.setf( ios::showpos );
	cout<<'\n'<<gPacket.Position()<<' '<<gPacket.Velocity()
	    <<'\n'<<gPacket.Attitude()<<' '<<gPacket.AngleRates()
	    <<endl;
    }

    cout<<endl;
}








