//========================================================================
// mathlib_class_Matrix.h
//
// 09 Dec 1999 Version 1.0
// 21 Aug 2000 version 1.1 - add vector<double>
// 28 May 2002         1.2 - add "struct":
//   				friend struct _row_matrix; 
//  				friend struct _col_matrix; 
//  				friend struct _sub_matrix; 
//
// This implements a simple matrix class which provides most common matrix
// arithematic operations.  This class also provides row, column, and subarray 
// operations.
//
// If checking is enabled, this class throws a MatrixError exception when
// subscripts are out of range.  
//
// NOTE: Checking is enabled by: matrix::check=true;
//
//  MatrixError members:
//
//  string _message;			// operation causing error
//  size_t _mtx1_row, _mtx1_col;	// first  matrix
//  size_t _mtx2_row, _mtx2_col;	// second matrix
//
//
//========================================================================

//page
/* -----------------------------------------------------------------------

-- Construction-
   A matrix can be constructed as follows:

 	matrix A (noRows, noCols=1);	
	matrix A (noRows, noCols, defaultValue);

	double array[3][3] = {11,12, 13, 21, 22, 23, 31,32,33};
	matrix A (noRows, noCols, array); 

        matrix A (noRows, noCols, vector<double> vectorList);

	matrix A = matrix(noRows, noCols).getarray(a11,a12,a13,a21,...);

-- Row and Column operations:

	A.row(a_row) = B;	      copy B matrix to row of A
	A.row(a_row) = C.row(c_row);  copy row of C to rwo of A
	D = F.row(f_row);   	      copy row of F to matrix D

	column operations are similar.
	

-- Subarray operations:

	A.subarray(begin_row,begin_col) = B;
				      copy matrix B to subarray of A

	C = A.subarray(begin_row, begin_col, no_rows, no_cols);
  				      copy subarray of A to matrix C

//page
-- Matrix operations:
	(capital letters are matrices, lower case are scalers)

	columns = A.colsize();	 	// get number of columns
	rows    = A.rowsize();		// get number of rows

	M = A.getarray (val, val,...);
      	M = A.getarray (ptr);		// get and put elements to c-style
	A.putarray (ptr);		// arrays- double X[i][j];

      	M = A.getarray (vector<double>& list); // 2nd subscript advances most 
	A.putarray (vector<double>& list);     // rapidly
	vector<double> = A.putarray();

        A = (*doublePtr)(c-style array);
        A = (vector<double>);

	s = A(row,col);	A(row,col) = s; // access element of matrix

	A = B ;				// assign one matrix to another  
	A = s ;				// assign scaler to all entries

	A = B + C;			// add two matricies
	A = B + s;  A = s + B;		// add scaler to each matrix element
	A =+B;				// monadic operator
	A +=B;
	A -=s;
					// same for subtraction

	A = B * C;			// matrix (row by column) multiply
	A = B * s;  A = s * B;		// multiply every element by a scaler				

	A = B / s; 			// divide each element by a scaler
	A /= s;

	A = B++;    A = transpose(B);	// matrix transpose

	A = B.identity();		// set B to identity and return B

	A = product(B,C);		// element by element multiply
	s = A.sum();			// sum of all elements

        s = A.max(&i,&j);		// maximum, minimum elements and size_t
        s = A.min(&i,&j);		// indicies.
	s = A.max();
	s = A.min();

	os<<A;				// print elements of matrix
*/
//------------------------------------------------------------------

//page
#if !defined(MATHLIB_MATRIX)
#define      MATHLIB_MATRIX

#include <string>
#include <vector>
#include <iostream>
using namespace std;

class matrix;


//------------------------------------------------------------------------
// MatrixError Exception Class
//
// This object is thrown for matrix indexing errors
//
//------------------------------------------------------------------------

struct MatrixError
{
    string _message;			// operation causing error
    size_t _mtx1_row, _mtx1_col;	// first  matrix
    size_t _mtx2_row, _mtx2_col;	// second matrix
    
    MatrixError 
    	(
	const string& message, 
	size_t mtx1row, 
	size_t mtx1col, 
	size_t mtx2row, 
	size_t mtx2col
	)
        : _message (message), 
	  _mtx1_row  (mtx1row), 
	  _mtx1_col  (mtx1col),
	  _mtx2_row  (mtx2row), 
	  _mtx2_col  (mtx2col)
	{}
};


//page
//------------------------------------------------------------------------
// _sub_matrix- internal auxillary structure
//
//   This is a helper class for implementing subarray operations such as:
//
//	 X.subarray(beginRow, beginCol) = Y;
//
//   which returns a _sub_matrix object on left side of the assign (=).
//   This structure stores a pointer to "X" matrix and adds the 
//   starting row and column.
//
//   The _sub_matrix::operator= then loads the X matrix with the 
//   Y elements. 
//   
//------------------------------------------------------------------------

struct _sub_matrix
{
    matrix& _mtx;
    size_t  _beginRow, _beginCol;
    
    _sub_matrix (matrix& mtx, size_t begRow,  size_t begCol);
    matrix& operator= (const matrix& m);
};


//------------------------------------------------------------------------
// _row_matrix- internal auxillary structure
//
//   This is a helper class for implementing subarray operations such as:
//
//	 X.row(xrow) = Y.row(yrow);
//
//   Each side of the assignment statement evaluates to a _row_matrix.
//
//   This structure stores a pointer to "X" or "Y" matrix and adds the 
//   row number. 
//
//   The _row_matrix::operator= then loads the X matrix with the 
//   Y elements. 
//
//------------------------------------------------------------------------

struct _row_matrix
{
    matrix& _mtx;
    size_t  _row;

    _row_matrix (matrix& mtx, size_t row);

    matrix& operator= (const matrix& m);
    matrix& operator= (const _row_matrix r);
};


//page
//------------------------------------------------------------------------
// _col_matrix- internal auxillary structure  (see _row_matrix notes)
//
//------------------------------------------------------------------------

struct _col_matrix
{
    matrix& _mtx;
    size_t  _col;

    _col_matrix (matrix& mtx, size_t col);

    matrix& operator= (const matrix& m);
    matrix& operator= (const _col_matrix r);
};


//
//------------------------------------------------------------------------
// Matrix Class- private members
//------------------------------------------------------------------------

class matrix
{

    friend struct _row_matrix;		// internal auxillary structures 
    friend struct _col_matrix;		// for implementing row, column and
    friend struct _sub_matrix;		// subarray operations.

    size_t  _rows;			// matrix number of rows, columns,
    size_t  _cols;			// and element storage.
    double *_arrayPtr;
   					// internal procedures
    void    _initial (size_t rows, size_t cols, void *ptr=0);
    double& _value   (size_t row,  size_t col) const;

public:					// Not actually intended for public use:
    matrix (const _row_matrix& rm); 	// internal constructors for row &
    matrix (const _col_matrix& rm);	// column operations.


//page
//=============================================================================
//
// PUBLIC Members:
// Matrix Class- constructors and destructors and initialization
//=============================================================================

public:
    static  bool check;
					// public constructors
    matrix (size_t rows, size_t cols=1, double value=0.0);
    matrix (const matrix& mtx);
    matrix (size_t rows, size_t cols, double list[]);
    matrix (size_t rows, size_t cols, const vector<double>& vector);

    matrix& identity();

    virtual ~matrix();

    void putarray (double list[]) const;
    void putarray (vector<double>& vec) const;

    vector<double> putarray() const; 

    matrix& getarray (const double list[]);
    matrix& getarray (const vector<double>& vec);
    matrix& getarray (double val, ...);

//------------------------------------------------------------------------
// Matrix Class- size  and index
//------------------------------------------------------------------------

public:
    size_t  rowsize () const {return _rows;}	// return row and
    size_t  colsize () const {return _cols;}	// column sizes

    double& operator() (size_t row, size_t col=1) const;


//page
//------------------------------------------------------------------------
// Matrix Class- Arithematic
//------------------------------------------------------------------------

public:
    matrix& operator= (double v);
    matrix& operator= (const matrix& m);
    matrix& operator= (const double list[]);
    matrix& operator= (const vector<double>& vec);

    friend matrix  operator+ (const matrix& m, double v);
    friend matrix  operator+ (double v, const matrix& m);
    friend matrix  operator+ (const matrix& m, const matrix& n);

    friend const matrix& operator+ (const matrix& m);
    friend matrix& operator+ (matrix& m); 	// monadic operator
    friend matrix  operator+=(matrix& m, const matrix& n);
    friend matrix  operator+=(matrix& m, double v);

    friend matrix  operator- (const matrix& m, double v);
    friend matrix  operator- (double v, const matrix& m);
    friend matrix  operator- (const matrix& m, const matrix& n);
    friend matrix  operator-=(matrix& m, const matrix& n);
    friend matrix  operator-=(matrix& m, double v);

    friend const matrix operator- (const matrix& m);
    friend matrix operator- (matrix& m);	// monadic operator

    friend matrix  operator* (const matrix& m, double v);
    friend matrix  operator* (double v, const matrix& m);
    friend matrix  operator* (const matrix& m, const matrix& n);
  
    friend matrix  operator/ (const matrix& m, double v);
    friend matrix  operator/=(matrix& m, double v);


//page
//------------------------------------------------------------------------
// Matrix Class- Other Operations
//------------------------------------------------------------------------

public:
    friend matrix  transpose  (const matrix& m);
    matrix  operator++ (int) const; // alternate transpose
    friend matrix  product    (const matrix& m, const matrix& n);



    double sum () const;	// sum of all elements
    double max (size_t *rowPtr=0, size_t  *colPtr=0) const; 
    double min (size_t *rowPtr=0, size_t  *colPtr=0) const;

			// row, column and subarray operations. (see notes)
    _row_matrix  row (size_t row);
    _col_matrix  col (size_t col);

     matrix     subarray (size_t row, size_t col, size_t noRows, size_t noCols) const;
    _sub_matrix subarray (size_t row, size_t col);

    friend ostream& operator<< (ostream& os, const matrix& m);
};

#endif

