//==============================================================================
// timelib_class_datetime.c- date and time class
//
// 03 Nov 1999 Version 1.0
//
// reference 1.  "Astronomical Algorithms", Jean Meeus, Willman-Bell, Inc., 1991
//
// reference 2.  "Almanac for Computers 1983", Nautical Almanac Office, United 
// States Naval Observatory, 1983.
//
//==============================================================================

#include <math.h>
#include <stdlib.h>
#include <sstream>
#include <iomanip>
#include "timelib_class_datetime.h"


//-----------------------------------------------------------------------------
// Month and Day Name Tables
//------------------------------------------------------------------------

const static char* monthList[13] = 
{
    0,
    "January"  , "February", "March"   , "April"  , 
    "May"      , "June"    , "July"    , "August" , 
    "September", "October" , "November", "December"
};


const static char* dayList[8] = 
{
    0, "Sunday", "Monday"  , "Tuesday", 
    "Wednesday", "Thursday", "Friday" , "Saturday"
};


//-----------------------------------------------------------------------------
// private procedure- initial internal variables
//
//------------------------------------------------------------------------

void date_time::_initial (double time)
{
    _time = time;
    _itime= 0.;
    _day  = 0.;
    _year = 0;
    _month= 0;
    _hour = 0;
    _min  = 0;
}


//page
//-----------------------------------------------------------------------------
// private procedure- get numeric equivalent of alphabetic month
//
//------------------------------------------------------------------------

size_t date_time::_GetNumericMonth (const string& month)
{
    for (int mon=1; mon<=12; mon++)
    {
        if  (strncasecmp (monthList[mon], month.c_str(), month.size()) ==0) 
	    return mon;
    }
 
    return 0;
}


//-----------------------------------------------------------------------------
// private procedure- Convert floating Day to day,hour,min,sec
//
//------------------------------------------------------------------------

long date_time::_DayToHourMinuteSecond 
(double day, size_t& hour, size_t& minute, double& second) const
{
    double wholeDay;
    second = modf(day, &wholeDay) * 86400.0L;
    
    hour   = (int)( second / 3600 );
    second-= hour * 3600;
    
    minute = (int)( second / 60   );
    second-= minute * 60;
    
    return (long)wholeDay; 
}


//page
//-----------------------------------------------------------------------------
// private procedure- Convert Julian Day to Gregorian Year,month,day
//
// 	This updates internal year,month,day values.  See ref. 1, page 63
//------------------------------------------------------------------------

void date_time::_GetGregorianYearMonthDay () const
{
    long   A;
    double Z;
    double F = modf (_time +0.5, &Z);
    
    if (Z<2299161)
    {
        A = (long)Z;
    }
    else
    {
        long a = (long)((Z-1867216.25L) / 36524.25L);
        A =  (long)(Z + 1 + a - a/4);
    }

    long B =  A + 1524;
    long C = (long)((B - 122.1L) / 365.25);
    long D = (long)(365.25L * C);
    long E = (long)((B - D) / 30.6001L);
    
    const_cast<date_time*>(this)->_day    =   B - D - (int)(30.6001L * E) + F ;
    const_cast<date_time*>(this)->_month  =  (E<14)     ? (E-1)    : (E-13)   ;
    const_cast<date_time*>(this)->_year   =  (_month>2) ? (C-4716) : (C-4715) ;

    const_cast<date_time*>(this)->_itime  = _time;
}

//page
//------------------------------------------------------------------------------
// Constructors
//
//------------------------------------------------------------------------

date_time::date_time () 
{
   _initial();
}

date_time::date_time (time_t time1970sec)
{
    _initial (time1970sec / 86400.L + 2440587.5L);
}

date_time::date_time (double julianDate)
{
    _initial (julianDate);
}

date_time::date_time (const date_time& datetime)
{
    _initial (datetime._time);
}


//page
//-----------------------------------------------------------------------------
// Arithmetic Operators
//
//------------------------------------------------------------------------

date_time date_time::operator+= (const double julianDays)
{
    _time += julianDays;
    return *this;
}

date_time date_time::operator-= (const double julianDays)
{
    _time -= julianDays;
    return *this;
}

double date_time::operator+ (double julianDays) const
{
    return _time + julianDays;
}    

double date_time::operator- (const date_time& datetime) const
{
    return _time - datetime._time;
}    

double date_time::operator- (double julianDays) const
{
    return _time - julianDays;
}    


//page
//------------------------------------------------------------------------------
// Comparison Operators
//
//------------------------------------------------------------------------

bool date_time::operator== (const date_time& datetime) const
{
    return _time == datetime._time;
} 

bool date_time::operator!= (const date_time& datetime) const
{
    return _time != datetime._time;
} 

bool date_time::operator<= (const date_time& datetime) const
{
    return _time <= datetime._time;
} 

bool date_time::operator>= (const date_time& datetime) const
{
    return _time >= datetime._time;
} 

bool date_time::operator> (const date_time& datetime) const
{
    return _time > datetime._time;
} 

bool date_time::operator< (const date_time& datetime) const
{
    return _time < datetime._time;
} 


//-----------------------------------------------------------------------------
// SetJulianDay
//
//------------------------------------------------------------------------

date_time date_time::SetJulianDay (double julianDay)
{
    _time = julianDay;
    return  *this;
}


//page
//------------------------------------------------------------------------------
// SetGregorianDate - (numeric month)
//
// Year may range from 1801 to 2099. Date is calender date. See ref. 2.
//
//------------------------------------------------------------------------------

date_time date_time::SetGregorianDate
    (size_t year, size_t month, size_t day, size_t hour, size_t minute, double second)
{
    size_t &K   = year;                    // Almanac Nomenclature
    size_t &M   = month;
    size_t &I   = day;

    double sign = (100*K+M - 190002.5L)<0 ? -1.0 : +1.0; 
    double UT   = hour + minute/60.0L + second/3600.0L;

    _time = 367*K - 7*(K+ ((M+9)/12)) / 4
    	          + (275*M)/9 + I + 1721013.5 + UT/24.0L 
    	          - 0.5L * sign + 0.5L;
    return *this;
}


//------------------------------------------------------------------------------
// SetGregorianDate - (alphbetic month)
//
//------------------------------------------------------------------------------

date_time date_time::SetGregorianDate
(size_t year, string month, size_t day, size_t hour, size_t minute, double second)
{
    size_t mon = _GetNumericMonth (month);
    SetGregorianDate (year, mon, day, hour, minute, second);
    return *this;
}


//page
//------------------------------------------------------------------------------
// SetGregorianYearMonthDay- (numeric month)
//
//------------------------------------------------------------------------

date_time date_time::SetGregorianYearMonthDay (size_t year, size_t month, double dayOfMonth)
{
    size_t hour, minute, day;
    double second;

    day = _DayToHourMinuteSecond (dayOfMonth, hour, minute, second);
    SetGregorianDate (year, month, day, hour, minute, second);

    return *this;
}


//------------------------------------------------------------------------------
// SetGregorianYearMonthDay- (alphabetic month)
//
//------------------------------------------------------------------------

date_time date_time::SetGregorianYearMonthDay (size_t year, string month, double dayOfMonth)
{
    size_t mon = _GetNumericMonth (month);
    SetGregorianYearMonthDay (year, mon, dayOfMonth);

    return *this;
}


//------------------------------------------------------------------------------
// SetGregorianYearDaySecond- first day is day 0
// 
//------------------------------------------------------------------------

date_time date_time::SetGregorianYearDaySecond (size_t year, size_t dayOfYear, double second)
{
    SetGregorianDate (year, 1, 1, 0, 0, 0.0);
    _time += (dayOfYear + second/86400.0L);

    return *this;
}


//------------------------------------------------------------------------------
// GetJulianDay
//
//------------------------------------------------------------------------

double date_time::GetJulianDay() const
{
    return _time;
}


//page
//------------------------------------------------------------------------------
// GetGregorianYearMonthDay- 	See ref. 1, page 63
//
//------------------------------------------------------------------------

void date_time::GetGregorianYearMonthDay (size_t& year, size_t& month, double& day) const
{
    if (_time != _itime) _GetGregorianYearMonthDay();

    day    = _day  ;
    month  = _month;
    year   = _year ;
}


//------------------------------------------------------------------------------
// GetGregorianDate
//
//------------------------------------------------------------------------


void date_time::GetGregorianDate 
       (
	size_t& year, size_t& month, size_t& day, size_t& hour, size_t& minute, double& second
       ) const
   {
	double floatDay;
    	GetGregorianYearMonthDay (year, month, floatDay);

        day = _DayToHourMinuteSecond (floatDay, hour, minute, second);
   }


//------------------------------------------------------------------------------
// GetGregorianYearDaySecond- first day is day 0
// 
//------------------------------------------------------------------------

void 
date_time::GetGregorianYearDaySecond (size_t& year, size_t& dayOfYear, double& second) const
   {
	size_t    month; 
	double    day  ;
	date_time temp ;

	GetGregorianYearMonthDay (year, month,  day);
	temp.SetGregorianYearDaySecond (year, 0, 0.);

	dayOfYear =  (size_t)(_time - temp._time);
	second    = (_time - (temp._time+dayOfYear)) * 86400;
   }


//page
//------------------------------------------------------------------------------
// GetDayofWeek- See ref. 1, page 65 (range is 1 to 7)
//
//------------------------------------------------------------------------

size_t date_time::GetNumericDayofWeek (size_t* day) const
{
     double  tmp = (long)(_time + 0.5L) - 0.5L;
     size_t  Day = ((long)(tmp + 1.5) % 7) + 1;
     if (day) *day = Day;
     return    Day;
}

//------------------------------------------------------------------------
//GetJulianDateAt0hr
//------------------------------------------------------------------------

double  date_time::GetJulianDateAt0hr (double* date) const
{
    double tmp = (long)(_time + 0.5L) - 0.5L;
    if (date) *date = tmp;
    return tmp;
}

//page
//------------------------------------------------------------------------------
// ConvertDateToString 
//------------------------------------------------------------------------------
string date_time::ConvertDateToString (const string format) const
{
    ostringstream dateStr;
    char    buf[4] = {0,0,0,0};
    size_t  yr, mon, day, hr, min;
    double  sec;
    size_t  dy;

    GetGregorianDate (yr, mon, day, hr, min, sec);

    for (size_t index = 0; index<format.length(); index++)
    {
	if (format[index] != '%') {
	    dateStr<< format[index];
	} 
	else {
	    char code1 = format[++index];
	    char code2 = format[++index];

	    switch (code1)
	    {
		case 'Y': 
		    if(code2=='4') dateStr<<setw(4)<<setfill('0')<<yr;
		    if(code2=='2') dateStr<<setw(2)<<setfill('0')<<yr%100;
		    break;
		case 'M': 	
		    if(code2=='n') dateStr<<setw(2)<<setfill('0')<<mon; 
		    if(code2=='A') dateStr<<monthList[mon];
		    if(code2=='a') dateStr<<strncpy(buf,monthList[mon],3);
		    break;
		case 'D':
		    if(code2=='n') dateStr<<setw(2)<<setfill('0')<<day; 
		    break;
		case 'W':
		    GetNumericDayofWeek(&dy);
		    if(code2=='n') dateStr<<dy;
		    if(code2=='A') dateStr<<dayList[dy];
		    if(code2=='a') dateStr<<strncpy(buf,dayList[dy],3);
		    break;
		case 'h':
		    if(code2=='h') dateStr<<setw(2)<<setfill('0')<<hr;
		    break;
		case 'm':
		    if(code2=='m') dateStr<<setw(2)<<setfill('0')<<min;
		    break;
		case 's':
		    dateStr<<setw(2)<<setfill('0')<<(int)sec;
		    break;
		case 'S':
		    dateStr<<setw(2)<<setfill('0')<<(int)sec;
		    dateStr<<'.'<<setw(3)<<setfill('0')<<(int)((sec-(int)sec)*1000);		    
	    }
	}
    }
    dateStr<<ends;
    return string( dateStr.str() );
}
//page


//------------------------------------------------------------------------------
// _CvtStr - (private procedure for ConvertStringToDate)
//
//------------------------------------------------------------------------------

static 
void _CvtStr (size_t& value, const string& date, size_t& index, size_t size)
{
   value = atoi( date.substr(index,size).c_str() );
   index += size;
}


static
void _CvtStr (double& value, const string& date, size_t& index, size_t size)
{
   value = atof ( date.substr(index,size).c_str() );
   index += size;
}


//page
//------------------------------------------------------------------------------
// ConvertStringToDate
//
//------------------------------------------------------------------------------

date_time& 
date_time::ConvertStringToDate( const string& dateStr, const string& format )
{
    size_t  idx = 0;
    size_t  yr  = 0, mon = 0, day = 0, hr = 0, min = 0;
    double  sec = 0;

    for (size_t index = 0; index<format.length(); index++)
    {
	if (format[index] != '%') {
	   idx++;
	} 
	else {
	    char code1 = format[++index];
	    char code2 = format[++index];

	    switch (code1)
	    {
		case 'Y': 
		    if(code2=='4') _CvtStr( yr,dateStr,idx,4 );
		    break;

		case 'M': 	
		    if(code2=='n') {_CvtStr( mon,dateStr,idx,2 ); break;}
			
		    mon = _GetNumericMonth( dateStr.substr(idx,3) );
		    if(code2=='A') {idx+=strlen(monthList[mon]); break;}
		    if(code2=='a') {idx+=3; break;}
		    break;

		case 'D':
		    if(code2=='n') _CvtStr( day,dateStr,idx,2 ); 
		    break;

		case 'h':
		    if(code2=='h') _CvtStr( hr,dateStr,idx,2 );
		    break;

		case 'm':
		    if(code2=='m') _CvtStr( min,dateStr,idx,2 );
		    break;

		case 'S':
		case 's':
		    _CvtStr( sec,dateStr,idx,code2-48 );
		    break;
	    }
	}
    }
    
    SetGregorianDate( yr, mon, day, hr, min, sec);
    return *this;
}


